const Main        = imports.ui.main;
const Shell       = imports.gi.Shell;
const GLib        = imports.gi.GLib;
const Meta        = imports.gi.Meta;
const Gio         = imports.gi.Gio;
const Utils       = imports.misc.extensionUtils.getCurrentExtension().imports.utils; 
const mySettings  = Utils.getSettings();
const ShellConfig = imports.misc.config;
const Mainloop    = imports.mainloop;
const St          = imports.gi.St;

const WA_COUNT          = 'count';
const WA_CHECKINTERVAL  = 'checkinterval';

let checkInterval, count, checking;
let button, extensionPath, buttonLabel, iconRead, iconUnread,iconSync;
let readCountPid, checkMailPid;

function notify_icon(msg, details, icon) {
    let source = new Main.MessageTray.Source("MyApp Information", icon);
    Main.messageTray.add(source);
    let notification = new Main.MessageTray.Notification(source, msg, details);
    notification.setTransient(true);
    source.notify(notification);
}


// Launch the default Mail application (thunderbird, evolution, ...)
function _RunMailApp() {
    let newCountstr =  mySettings.get_string(WA_COUNT);
	let newCount    =  parseInt(newCountstr);

	if (newCount < 0) { _checkKeyring(); }
	else {
        try {
            let appinfo = Gio.app_info_get_default_for_type("x-scheme-handler/mailto", false);
            let success = appinfo.launch([], null);
            if (!success) {
				Main.notify('Error launching default MUA');
            }
        } catch (err) {
			Main.notify('Can not start default MUA: '+err.message);
        }
    }
}



function _readCount() {
    try {
        let newCountstr =  mySettings.get_string(WA_COUNT);
	    let newCount    =  parseInt(newCountstr);
	    
	    // Update icon and notify only if needed
        if (newCount != count) {
	        if (newCount > count) {
		        if (newCount == 1) {   notify_icon("Mail:", "You have received a new email!", 'imap-notify-unread-symbolic');          };
		        if (newCount >  1) {   notify_icon("Mail:", 'You have received '+newCount+' emails!', 'imap-notify-unread-symbolic');  };
	        }

	        if (newCount == -1)    {   notify_icon("GNOME Keyring:", "GNOME Keyring is locked", 'imap-notify-lock-symbolic');}
	        if (newCount == -2)    {   notify_icon("Error:", "Unable to connect to imap server", 'imap-notify-read-symbolic');}
	        
	        // Update icon 
            _setMessageCount(newCount);           
        }
        
    } catch (err) { Main.notify('Error: Can not read message count: ' + err.message); }
    
    return true;
}

function _checkMail() {
    if (!checking) {
        checking = true;
        let command = extensionPath + "/check_mail";
        try {
            let killall = "/usr/bin/killall";
            let [success, pid] = GLib.spawn_sync(null, [killall, command], null, 0, null, null, null, null, null);
            if (!success) { Main.notify('Error killing check_mail'); }
            GLib.spawn_close_pid(pid);
        } catch (err) {
			Main.notify('Error killing check_mail:\n'+err.message);
        }
        try {
            let [success, pid] = GLib.spawn_async(null, [command], null, 0, null);
            if (!success) { Main.notify('Error executing check_mail'); }
            GLib.spawn_close_pid(pid);
        } catch (err) { Main.notify('Error executing check_mail:\n'+err.message); }
        
        
        checking = false;
    }
    return true;
}


function _checkKeyring() {
	let cmd = extensionPath + "/check_keyring";

    try {
        Main.Util.trySpawnCommandLine('/bin/chmod +x ' + cmd);
    } catch(err) {
        Main.notify('Error: Can not chmod  ' + cmd);
    }
    try {
        Main.Util.trySpawnCommandLine(cmd);
    } catch(err) {
        Main.notify('Error: ' + cmd + 'has failed');
    }
    return true;
    
}



function _setMessageCount(newCount) {
    count = newCount;
    if (count == 0) {
        iconSync.hide();
        iconLock.hide();
        iconUnread.hide();
        buttonLabel.hide();
        iconRead.show();
    }

    if (count > 0) {
        buttonLabel.text = count.toString();
        iconSync.hide();
        iconLock.hide();
        iconRead.hide();
        iconUnread.show();
        buttonLabel.show();
    }
    if (count == -1) {
        iconSync.hide();
        iconUnread.hide();
        iconRead.hide();
        buttonLabel.hide();
        iconLock.show();
    }
    if (count == -2) {
        iconLock.hide();
        iconRead.hide();
        iconUnread.hide();
        buttonLabel.hide();
        iconSync.show();
    }
}


function init(meta) {
    extensionPath = meta.path;

    let theme = imports.gi.Gtk.IconTheme.get_default();
    theme.append_search_path(extensionPath + "/icons");

    count = -3;
    checking = false;
    
    let checkIntervalstr =  mySettings.get_string(WA_CHECKINTERVAL);
	checkInterval        =  parseInt(checkIntervalstr);
	
	// Set MinMax for checkInterval [5;30]
    if (checkInterval<5) {checkInterval = 5;};
    if (checkInterval>20){checkInterval = 30;};

    button     = new St.Bin({ style_class: 'panel-button', reactive: true, can_focus: true, x_fill:true, y_fill: false, track_hover:true });

    iconRead   = new St.Icon({ icon_name: 'imap-notify-read-symbolic'           , style_class: 'system-status-icon' });
    iconUnread = new St.Icon({ icon_name: 'imap-notify-unread-symbolic'         , style_class: 'system-status-icon' });
    iconLock   = new St.Icon({ icon_name: 'imap-notify-lock-symbolic'           , style_class: 'system-status-icon' });
    iconSync   = new St.Icon({ icon_name: 'imap-notify-synchronizing-symbolic'  , style_class: 'system-status-icon' });

    iconLock.hide();
    iconRead.hide();
    iconUnread.hide();
    
    buttonLabel = new St.Label({ style_class: 'button_text', text: '...' });
    var box = new St.BoxLayout();
     
    box.insert_child_at_index(iconSync   , 1);
    box.insert_child_at_index(iconLock   , 2);
    box.insert_child_at_index(iconRead   , 3);
    box.insert_child_at_index(iconUnread , 4);
    box.insert_child_at_index(buttonLabel, 5);
    
    button.set_child(box);
    
    button.connect('button-press-event', _RunMailApp);

    // Enable execute Script chmod
    imports.misc.util.trySpawnCommandLine('/bin/chmod +x ' + extensionPath + '/check_keyring');
    imports.misc.util.trySpawnCommandLine('/bin/chmod +x ' + extensionPath + '/check_mail');
 
}

function enable() {
    Main.panel._rightBox.insert_child_at_index(button, 0);
    checkMailPid = GLib.timeout_add_seconds(0, checkInterval, _checkMail);
    readCountPid = GLib.timeout_add_seconds(0, checkInterval, _readCount);
}

function disable() {
    Main.panel._rightBox.remove_actor(button);
    if (readCountPid != null) {
        Mainloop.source_remove(readCountPid);
        readCountPid = null;
    }
    if (checkMailPid != null) {
        Mainloop.source_remove(checkMailPid);
        checkMailPid = null;
    }
}
